# -*- coding: utf-8 -*-

"""Constants for third-party applications."""

from __future__ import annotations

__all__ = ['EMPTY_PATCH']

import typing

from . import base


EMPTY_PATCH: typing.Final[base.AppPatch] = base.AppPatch()
